
local MakePlayerCharacter = require "prefabs/player_common"


local assets = {
    Asset("SCRIPT", "scripts/prefabs/player_common.lua"),
}
local prefabs = {}

-- Custom starting items
local start_inv = { --"bloodwhip"
}

-- When the character is revived from human
local function onbecamehuman(inst)
	-- Set speed when reviving from ghost (optional)
	inst.components.locomotor:SetExternalSpeedMultiplier(inst, "dracul_speed_mod", 1)
end

local function onbecameghost(inst)
	-- Remove speed modifier when becoming a ghost
   inst.components.locomotor:RemoveExternalSpeedMultiplier(inst, "dracul_speed_mod")
end

-- When loading or spawning the character
local function onload(inst)
    inst:ListenForEvent("ms_respawnedfromghost", onbecamehuman)
    inst:ListenForEvent("ms_becameghost", onbecameghost)

    if inst:HasTag("playerghost") then
        onbecameghost(inst)
    else
        onbecamehuman(inst)
    end
end
	local NIGHTVISION_COLOURCUBES =
{
    day = "images/colour_cubes/insane_day_cc.tex",
    dusk = "images/colour_cubes/insane_dusk_cc.tex",
    night = "images/colour_cubes/purple_moon_cc.tex",
    full_moon = "images/colour_cubes/purple_moon_cc.tex",
}

local function SetNightVision(inst, enable)
    if TheWorld.state.isnight or TheWorld:HasTag("cave") then
        inst.components.playervision:ForceNightVision(true)
        inst.components.playervision:SetCustomCCTable(NIGHTVISION_COLOURCUBES)
    else
        inst.components.playervision:ForceNightVision(false)
        inst.components.playervision:SetCustomCCTable(nil)
    end
end

-- This initializes for both the server and client. Tags can be added here.
local common_postinit = function(inst) 
	-- Minimap icon
	inst.MiniMapEntity:SetIcon( "dracul.tex" )
	
	--the following goes into the commonpostinit, that being the code that is run on clients--
--This is sort of messy and gross but it works. 
inst:WatchWorldState( "isday", function() SetNightVision(inst) end)
  	inst:WatchWorldState( "isdusk", function() SetNightVision(inst) end)
  	inst:WatchWorldState( "isnight", function() SetNightVision(inst)  end)
	inst:WatchWorldState( "iscaveday", function() SetNightVision(inst) end)
  	inst:WatchWorldState( "iscavedusk", function() SetNightVision(inst) end)
  	inst:WatchWorldState( "iscavenight", function() SetNightVision(inst)  end)
	
	
SetNightVision(inst) 
end

	--Dracul's Day Cycle Stats--
local function updatestats(inst)

	if TheWorld.state.phase == "day" then
		
		inst.Light:Enable(false)
		inst.components.hunger:SetRate(TUNING.WILSON_HUNGER_RATE*1) --No Regen, Increased Hunger, Decresed Health, Decresed Speed, Decreased Attack 
		inst.components.health:StartRegen(0, 0)
		inst.components.combat.damagemultiplier = 0.8

		local health_percent = inst.components.health:GetPercent()
	    inst.components.health:SetMaxHealth(100)
	    inst.components.health:SetPercent(health_percent, true)
		
		local hunger = inst.components.hunger:GetPercent()
		inst.components.hunger:SetMax(150)
		inst.components.hunger:SetPercent(hunger, true)

		inst.components.locomotor.walkspeed = 4
		inst.components.locomotor.runspeed = 8
		
	elseif TheWorld.state.phase == "dusk" then
		
		inst.Light:Enable(false)
		inst.components.hunger:SetRate(TUNING.WILSON_HUNGER_RATE*1.5) --Slow Regen, Decreased Hunger, Increased Health, Increased Speed, Increased Attack
		inst.components.health:StartRegen(1, 6)  --1Hp, 6 seconds. 10hp/min.
		inst.components.combat.damagemultiplier = 1.5

		local health_percent = inst.components.health:GetPercent()
	    inst.components.health:SetMaxHealth(200)
	    inst.components.health:SetPercent(health_percent, true)
		
		local hunger = inst.components.hunger:GetPercent()
		inst.components.hunger:SetMax(100)
		inst.components.hunger:SetPercent(hunger, true)

		inst.components.locomotor.walkspeed = 6
		inst.components.locomotor.runspeed = 8
	
	
	elseif TheWorld.state.phase == "night" then
		
		inst.Light:Enable(false)
		inst.components.health:StartRegen(2, 3)  --1Hp, 3 seconds. 20hp/min. --Fast Regen, Greater Decreased Hunger, Greater Increased Health, Greater Increased Speed, Greater Increased Atack
		inst.components.hunger:SetRate(TUNING.WILSON_HUNGER_RATE*2)
		inst.components.combat.damagemultiplier = 2

		local health_percent = inst.components.health:GetPercent()
	    inst.components.health:SetMaxHealth(333)
	    inst.components.health:SetPercent(health_percent, true)
		
		local hunger = inst.components.hunger:GetPercent()
		inst.components.hunger:SetMax(80)
		inst.components.hunger:SetPercent(hunger, true)

		inst.components.locomotor.walkspeed = 8
		inst.components.locomotor.runspeed = 10.8
	
		if TheWorld.state.isfullmoon then
			
			inst.components.health:StartRegen(5, 2.5)  --2Hp, .75 seconds. 160hp/min. --Double Regen, Double Decreased Hunger, Double Increased Health, Double Increased Speed, Double Increased Attack
			inst.components.hunger:SetRate(3*TUNING.WILSON_HUNGER_RATE)
			inst.components.combat.damagemultiplier = 6

			local health_percent = inst.components.health:GetPercent()
		    inst.components.health:SetMaxHealth(666)
		    inst.components.health:SetPercent(health_percent, true)
			
			local hunger = inst.components.hunger:GetPercent()
			inst.components.hunger:SetMax(50)
			inst.components.hunger:SetPercent(hunger, true)

			inst.components.locomotor.walkspeed = 12
			inst.components.locomotor.runspeed = 12
			inst.components.talker:Say("The night is young. Time to feed!")
		end
	end
end	

-- This initializes for the server only. Components are added here.
local master_postinit = function(inst)
	-- choose which sounds this character will play
	inst.soundsname = "dracul" --Violin Music
	-- Uncomment if "wathgrithr"(Wigfrid) or "webber" voice is used
    --inst.talker_path_override = "dontstarve_DLC001/characters/"
	
	--Not afraid of monsters
	inst.components.sanity.neg_aura_mult = 0
	
	--Hard to become insane
	inst.components.sanity.dapperness = TUNING.DAPPERNESS_HUGE
	
	--Able to eat anything even monster meat
	inst.components.eater.strongstomach = true
	inst.components.eater.ignoresspoilage = true
	
	-- Watches for changes in the day/dusk/night
	inst:WatchWorldState("phase",updatestats)
	inst:WatchWorldState("isfullmoon",updatestats)
	updatestats(inst)
	
end

return MakePlayerCharacter("dracul", prefabs, assets, common_postinit, master_postinit, start_inv)